#!/usr/bin/env bash
##
## Copyright (C) HySoP 2011-2022
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##         http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
set -feu -o pipefail

# /hysop should be mounted as read only by run_tests_in_docker.sh
if [[ ! -d '/hysop' ]]; then
    echo "This script should not be called from host, but from within a docker image."
    echo " => /hysop has not been mounted (see hysop/ci/utils/run_ci.sh)."
    exit 1
fi

CC=gcc
CXX=g++
FC=gfortran

HYSOP_DIR='/tmp/hysop'
HYSOP_BUILD_DIR="${HYSOP_DIR}/build"
HYSOP_INSTALL_DIR='/opt/hysop'

cp -r /hysop "${HYSOP_DIR}"
rm -rf "${HYSOP_BUILD_DIR}"

SCRIPT_DIR="${HYSOP_DIR}/ci/scripts"
cd "${HYSOP_DIR}"
${SCRIPT_DIR}/version.sh
${SCRIPT_DIR}/config.sh "${HYSOP_BUILD_DIR}" "${HYSOP_INSTALL_DIR}" "${CC}" "${CXX}" "${FC}"
${SCRIPT_DIR}/build.sh "${HYSOP_BUILD_DIR}" "${CC}" "${CXX}" "${FC}"
${SCRIPT_DIR}/install.sh "${HYSOP_BUILD_DIR}" "${HYSOP_INSTALL_DIR}"
time ${SCRIPT_DIR}/test.sh "${HYSOP_INSTALL_DIR}" "${HYSOP_DIR}/hysop"

# clean everything because image may be commited to retain hysop cache
cd
rm -rf "${HYSOP_DIR}" "${HYSOP_INSTALL_DIR}"
