#!/bin/bash
##
## Copyright (C) HySoP 2011-2022
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##         http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
set -feu -o pipefail

PYTHON_EXECUTABLE=${PYTHON_EXECUTABLE:-"$(which python3.9)"}

if [ $# -ne 2 ]; then
    echo "Usage ./install build_folder install_folder"
    exit 1
fi

if [ ! -d "$1" ]; then
    echo "Build folder $1 does not exist."
    exit 1
fi

if [ -d "$2" ]; then
    echo "Install folder $2 already exists."
    exit 1
fi

BUILD_FOLDER="$1"
INSTALL_FOLDER="$2"

cd "${BUILD_FOLDER}"
make install

if [ ! -d "${INSTALL_FOLDER}/lib/python3.9/site-packages/hysop" ]; then
    echo "${INSTALL_FOLDER}/lib/python3.9/site-packages/hysop was not created."
    exit 1
fi

export PYTHONPATH="${INSTALL_FOLDER}/lib/python3.9/site-packages"
"${PYTHON_EXECUTABLE}" -c 'import hysop; print(hysop)'

exit 0
